# %% Imports
import gradio as gr
import numpy as np
import os
import pandas as pd
import socket

# %% Plotting Function
def make_sine_wave(a=1, offset=0, f=3, phi=0):
    t = np.linspace(0, 2, 200)
    y = a * np.sin(2 * np.pi * f * t + phi) + offset

    return pd.DataFrame({'t': t, 'y': y})

# %% Interface Layout
with gr.Blocks() as demo:
    with gr.Row(equal_height=True):
        with gr.Column(scale=1, min_width=200):
            amp = gr.Slider(minimum=-5, maximum=5, step=0.1, value=1, label="Amplitude")
            offset = gr.Slider(minimum=-5, maximum=5, step=0.1, value=0, label="Offset")
            freq = gr.Slider(minimum=-5, maximum=5, step=0.01, value=1, label="Frequency")
            phase = gr.Slider(minimum=-3.1416, maximum=3.1416, step=0.1, value=0, label="Phase")

        # should work, but does not:
        # plot = gr.LinePlot(make_sine_wave, inputs=[amp, offset, freq, phase], x='t', y='y', title='Sine Wave')
        # as alternative initialize plot and trigger updates using gr.on:
        plot = gr.LinePlot(make_sine_wave(), x='t', y='y', title='Sine Wave')

    gr.on(
        triggers=[amp.change, offset.change, freq.change, phase.change, demo.load],
        fn=make_sine_wave,
        inputs=[amp, offset, freq, phase],
        outputs=plot,
    )

# %% Start Gradio App
## JupyterHub or anywhere else, public link, max. 72 h
# demo.launch(share=True)

## JupyterHub, proxy link
user = os.environ.get('JUPYTERHUB_USER')
port = 7860
print(f'Run this in (non-interactive) terminal. Stop server with Ctrl+C. Then you can reuse the same port.\nAccess WebGUI server on https://jupyter.hs-bochum.de/user/{user}/proxy/{port}/')
demo.launch(server_port=port, root_path=f'/user/{user}/proxy/{port}/', quiet=True)

## local network, allow access via network IP
# s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
# s.connect(("8.8.8.8", 80))
# my_ip = s.getsockname()[0]
# s.close()
# port = 7860
# print(f'Run this in (non-interactive) terminal. Stop server with Ctrl+C. Then you can reuse the same port.\nAccess WebGUI server on https://{my_ip}:7860')
# demo.launch(server_name='0.0.0.0', server_port=port, quiet=True)

## local computer, access only allowed from the computer itself, works also in VS Code's interactive window
# demo.launch()
